package com.stellvoy.user.controller;

import com.stellvoy.api.client.learning.LearningClient;
import com.stellvoy.api.dto.leanring.LearningInterestDTO;
import com.stellvoy.api.dto.leanring.LearningMedalDTO;
import com.stellvoy.api.dto.user.LoginFormDTO;
import com.stellvoy.api.dto.user.NewUserDTO;
import com.stellvoy.common.domain.dto.LoginUserDTO;
import com.stellvoy.common.exceptions.BadRequestException;
import com.stellvoy.common.utils.BeanUtils;
import com.stellvoy.user.constants.UserErrorInfo;
import com.stellvoy.user.domain.dto.*;
import com.stellvoy.user.domain.po.*;
import com.stellvoy.user.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.beans.Transient;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
@RestController
@RequestMapping("/users")
@Slf4j
public class UserController {

    @Autowired
    private IUserService userService;
    @Autowired
    private ICodeService codeService;
    @Autowired
    private UserMedalService userMedalService;
    @Autowired
    private UserInterestService userInterestService;
    @Autowired
    private LearningClient learningClient;
    private static final String TOKEN = "7zN4K2xQ8mP5L1rT9B3vF6z0H7jJ2mQ8"; // 你在微信公众平台配置的 token

    /**
     * 根据id查询用户信息
     */
    @GetMapping("/{id}")
    public NewUserDTO queryUserById(@PathVariable("id") Long id) {
        User one = userService.lambdaQuery().eq(User::getId, id).one();
        return BeanUtils.copyBean(one, NewUserDTO.class);
    }
    /**
     * 根据id集合查询用户信息
     */
    @GetMapping("/list/{ids}")
    public List<NewUserDTO> queryUserByIds(@PathVariable("ids") List<Long> ids) {
        List<User> userList = userService.lambdaQuery().in(User::getId, ids).list();
        return BeanUtils.copyList(userList, NewUserDTO.class);
    }
    /**
     * 登录结构
     * @param loginDTO 登录表单
     * @param isStaff 是否是后台登录
     * @return 登录用户信息
     */
    @PostMapping("/detail/{isStaff}")
    public LoginUserDTO queryUserDetail(
            @Valid @RequestBody LoginFormDTO loginDTO, @PathVariable("isStaff") boolean isStaff) {
        return userService.queryUserDetail(loginDTO, isStaff);
    }

    /**
     * 获取手机登录验证码
     */
    @GetMapping("/number/{phone}")
    public void getNumber(
            @PathVariable("phone") String cellPhone) {
        codeService.sendVerifyCode(cellPhone);
    }
    /**
     * 验证验证码
     */
    @PostMapping("/weChatVerify/{userId}")
    public void weChatVerifyCode(@RequestBody WeChatVerifyCodeDTO dto, @PathVariable("userId") Long userId) {
        userService.weChatVerifyCode(dto,userId);
    }

    @GetMapping("/list/ids")
    public Long exchangeUserIdWithPhone(@RequestParam("phone") String phone) {
        User user = userService
                .lambdaQuery().eq(User::getCellPhone, phone).one();
        if (user == null) {
            throw new BadRequestException(UserErrorInfo.Msg.USER_ID_NOT_EXISTS);
        }
        return user.getId();
    }

    /**
     * 根据用户ID查询全部勋章
     */
    @GetMapping("/medal/{userId}")
    public List<LearningMedalDTO> queryUserMedal(@PathVariable("userId") Long userId) {
        List<Long> medalIds = userMedalService.lambdaQuery()
                .eq(UserMedal::getUserId, userId)
                .list()
                .stream()
                .map(UserMedal::getMedalId)
                .collect(Collectors.toList());
        List<LearningMedalDTO> dtoList = learningClient.queryMedal();
        for (LearningMedalDTO learningMedalDTO : dtoList) {
            if(medalIds.contains(learningMedalDTO.getId())){
                learningMedalDTO.setType(1);
            }
        }
        return dtoList;
    }

    /**
     * 根据用户ID查询全部兴趣标签
     */
    @GetMapping("/interest/{userId}")
    public List<LearningInterestDTO> queryUserInterest(@PathVariable("userId") Long userId) {
        List<Long> interestIds = userInterestService.lambdaQuery()
                .eq(UserInterest::getUserId, userId)
                .list()
                .stream()
                .map(UserInterest::getInterestId)
                .collect(Collectors.toList());
        List<LearningInterestDTO> dtoList = learningClient.queryInterest();
        for (LearningInterestDTO learningInterestDTO : dtoList) {
            if(interestIds.contains(learningInterestDTO.getId())){
                learningInterestDTO.setType(1);
            }
        }
        return dtoList;
    }

    /**
     * 实名认证
     */
    @PostMapping("/verify/{userId}")
    public void verifyCode(@RequestBody VerifyCodeDTO dto, @PathVariable("userId") Long userId) {
        userService.verifyCode(dto,userId);
    }

    /**
     * 修改登录密码
     */
    @PostMapping("/password/{userId}")
    public void updatePassword(@RequestBody UpdatePasswordDTO dto, @PathVariable("userId") Long userId) {
        userService.updatePassword(dto,userId);
    }

    /**
     * 绑定邮箱
     */
    @PostMapping("/email/{userId}")
    public void updateEmail(@RequestBody UpdateEmailDTO dto, @PathVariable("userId") Long userId) {
        userService.updateEmail(dto,userId);
    }

    /**
     * 邮箱验证码
     */
    @GetMapping("/email")
    public void sendEmailCode(@RequestParam String email) {
        codeService.sendEmailCode(email);
    }

    /**
     * 修改用户兴趣标签
     */
    @Transient
    @PostMapping("/updateInterest/{userId}")
    public void updateInterest(@RequestBody List<Long> interestIds, @PathVariable("userId") Long userId) {
        // 先删除指定用户的所有兴趣标签记录
        userInterestService.lambdaUpdate()
                .eq(UserInterest::getUserId, userId)
                .remove();

        // 批量插入新的兴趣标签记录
        if (interestIds != null && !interestIds.isEmpty()) {
            List<UserInterest> newUserInterests = interestIds.stream()
                    .map(interestId -> {
                        UserInterest userInterest = new UserInterest();
                        userInterest.setUserId(userId);
                        userInterest.setInterestId(interestId);
                        return userInterest;
                    })
                    .collect(Collectors.toList());

            userInterestService.saveBatch(newUserInterests);
        }
    }

    /**
     *修改isNew
     */
    @PostMapping("/isNew/{userId}")
    public void isNew(@PathVariable("userId") Long userId) {
        userService.lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getIsNew, 1)
                .update();
    }

    /**
     * 统计用户任务数量
     * @param userId
     * @return
     */
    @GetMapping("/stats/{userId}")
    public UserTaskDTO getStats(@PathVariable Long userId) {
        return userService.getStats(userId);
    }
    /**
     * 上传个人简历
     */
    @PostMapping("/resume/{userId}")
    public void uploadResume(@RequestBody String resume, @PathVariable("userId") Long userId) {
        if (resume == null || resume.length() < 2) {
            throw new IllegalArgumentException("简历内容无效");
        }
        String trimmedResume = resume.substring(1, resume.length() - 1);
        userService.lambdaUpdate().eq(User::getId, userId)
                .set(User::getResume, trimmedResume)
                .update();
    }



    @GetMapping("/wechat/callback")
    public void wechatCallback(@RequestParam String signature,
                               @RequestParam String timestamp,
                               @RequestParam String nonce,
                               @RequestParam String echostr,
                               HttpServletResponse response) {
        log.info("微信回调参数：signature={}, timestamp={}, nonce={}, echostr={}", signature, timestamp, nonce, echostr);

        // 设置响应内容类型为 text/plain
        response.setContentType("text/plain;charset=UTF-8");

        // 验证签名
        if (checkSignature(signature, timestamp, nonce)) {
            // 验证成功，返回 echostr
            try {
                response.getWriter().write(echostr);
            } catch (IOException e) {
                log.error("写入响应时发生错误", e);
                throw new RuntimeException("写入响应时发生错误", e);
            }
        } else {
            // 验证失败，返回任意内容
            try {
                response.getWriter().write("验证失败");
            } catch (IOException e) {
                log.error("写入响应时发生错误", e);
                throw new RuntimeException("写入响应时发生错误", e);
            }
        }
    }

    private boolean checkSignature(String signature, String timestamp, String nonce) {
        String[] arr = new String[]{TOKEN, timestamp, nonce};
        Arrays.sort(arr);
        StringBuilder content = new StringBuilder();
        for (String s : arr) {
            content.append(s);
        }
        String tmp = sha1(content.toString());
        log.info("计算签名: {}", tmp);
        log.info("微信签名: {}", signature);
        return tmp.equals(signature);
    }

    private String sha1(String str) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-1");
            byte[] result = digest.digest(str.getBytes());
            StringBuilder sb = new StringBuilder();
            for (byte b : result) {
                sb.append(String.format("%02x", b));
            }
            return sb.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }





    /**
     * 生成微信登录二维码
     */
    @GetMapping("/wechat/login")
    public WeChatDTO wechatLogin() throws IOException {
        return userService.wechatLogin();
    }

    /**
     * 处理微信回调，接收 code
     */
    @GetMapping("/wechat/code")
    public void wechatCallback(@RequestParam String code,
                               @RequestParam String state){
        userService.wechatCallback(code, state);
    }
    /**
     * 处理前端轮询
     */
    @GetMapping("/wechat/login/status")
    public CheckLoginDTO checkLoginStatus(@RequestParam String state){
        log.info("state={}",state);
        return userService.checkLoginStatus(state);
    }

}
